/* dist for Blind Galaxy
 *
 * Program that prints a list of planets that are at a distance less than a
 * given limit from a given planet
 *
 * (c) 1997 Laurent Desnogues (desnogue@aiguemarine.unice.fr)
 */

#include <stdio.h>
#include <string.h>
#include <math.h>

#define MAX_PLANETS 1000
#define MAX_NAME_LG  100
#define MAX_LINE_LG 1000

typedef struct {
  char   name[MAX_NAME_LG];
  double x, y;
} t_planet;

t_planet planets[MAX_PLANETS];
int      nb_pl;

double min(double x, double y)
{
  return x > y  ? y : x;
}

void read_planets(FILE *f)
{
  char  buf[MAX_LINE_LG];
  int   nb;

  nb_pl = 0;
  while (1) {
    fgets(buf, MAX_LINE_LG, f);
    if (feof(f))
      break;
    nb = sscanf(buf, "%s%lf%lf",
		planets[nb_pl].name, &planets[nb_pl].x, &planets[nb_pl].y);
    if (nb != 3) {
      fprintf(stderr, "Invalid planet file\n`%s' -> %d\n", buf, nb);
      exit(2);
    }
    nb_pl++;
  }
  fprintf(stderr, "Read %d planets\n", nb_pl);
}

int find_planet(char *name)
{
  int i;

  for (i = 0; i < nb_pl; i++)
    if (!strcasecmp(name, planets[i].name))
      return i;
  return -1;
}

double dist_planets(double size, int p1, int p2)
{
  double dx, dy;

  dx = min(fabs(planets[p1].x - planets[p2].x),
	   size - fabs(planets[p1].x - planets[p2].x));
  dy = min(fabs(planets[p1].y - planets[p2].y),
	   size - fabs(planets[p1].y - planets[p2].y));
  return sqrt(dx * dx + dy * dy);
}

void main(int argc, char *argv[])
{
  int     pl;
  double  dist_max;
  double  dist;
  double  size;
  char   *end;
  int     i;
  FILE   *f_in;

  if (argc == 5) {
    f_in = fopen(argv[4], "r");
    if (f_in == NULL) {
      fprintf(stderr, "Could not open `%s'\n", argv[4]);
      exit(-1);
    }
  } else if (argc != 4) {
    fprintf(stderr, "Usage:  %s size planet dist [planets_file]\n", argv[0]);
    exit(1);
  } else
    f_in = stdin;

  read_planets(f_in);
  pl = find_planet(argv[2]);
  if (pl == -1) {
    fprintf(stderr, "Did not find planet `%s'\n", argv[1]);
    exit(3);
  }

  dist_max = strtod(argv[3], &end);
  if (*end != '\0') {
    fputs("Expect an FP number as second argument\n", stderr);
    exit(1);
  }

  size = strtod(argv[1], &end);
  if (*end != '\0' || size <= 0.0) {
    fputs("Expect a positive FP number as third argument\n", stderr);
    exit(1);
  }

  printf("Planets at a distance <= %.2f from planet `%s' (%.2f, %.2f):\n",
	 dist_max, planets[pl].name, planets[pl].x, planets[pl].y);
  for (i = 0; i < nb_pl; i++)
    if (i != pl && (dist = dist_planets(size, pl, i)) <= dist_max)
      printf("\t%s\t%+7.2f\t%+7.2f\t%6.2f\n",
	     planets[i].name, planets[i].x, planets[i].y, dist);

  exit(0);
}
